// Copyright (C) 2004-2007 Tord Romstad

// This file is part of Glaurung.
//
// Glaurung is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//  
// Glaurung is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Glaurung; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


// The code in this file is based on the opening book code in PolyGlot
// by Fabien Letouzey.  PolyGlot is available under the GNU Public License,
// and can be downloaded from http://wbec-ridderkerk.nl


#if !defined(BOOK_H_INCLUDED)
#define BOOK_H_INCLUDED


////
//// Includes
////

#include <string>

#include "move.h"
#include "position.h"


////
//// Types
////

struct BookEntry {
  uint64 key;
  uint16 move;
  uint16 count;
  uint16 n;
  uint16 sum;
};

class Book {

public:
  // Constructors
  Book();

  // Open and close book files
  void open(const std::string& fileName);
  void close();

  // Testing if a book is opened
  bool is_open() const;

  // Get a book move for a given position
  Move get_move(const Position& pos) const;

  // Print the book moves for a given position (for debugging)
  void print_moves(const Position& pos) const;

  // Print the book key for a given position (for debugging)
  void print_key(const Position& pos) const;

private:
  int find_key(uint64 key) const;
  void read_entry(BookEntry& entry, int n) const;

  FILE* bookFile;
  int bookSize;
};


////
//// Global variables
////

extern Book OpeningBook;


#endif // !defined(BOOK_H_INCLUDED)
