// Copyright (C) 2004-2007 Tord Romstad

// This file is part of Glaurung.
//
// Glaurung is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//  
// Glaurung is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Glaurung; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


////
//// Includes
////

#include "piece.h"


////
//// Constants and variables
////

const int SlidingArray[18] = {
  0, 0, 0, 1, 2, 3, 0, 0, 0, 0, 0, 1, 2, 3, 0, 0, 0, 0
};

const SquareDelta Directions[16][16] = {
  {DELTA_ZERO},
  {DELTA_NW, DELTA_NE, DELTA_ZERO},
  {DELTA_SSW, DELTA_SSE, DELTA_SWW, DELTA_SEE,
   DELTA_NWW, DELTA_NEE, DELTA_NNW, DELTA_NNE, DELTA_ZERO},
  {DELTA_SE, DELTA_SW, DELTA_NE, DELTA_NW, DELTA_ZERO},
  {DELTA_S, DELTA_E, DELTA_W, DELTA_N, DELTA_ZERO},
  {DELTA_S, DELTA_E, DELTA_W, DELTA_N, 
   DELTA_SE, DELTA_SW, DELTA_NE, DELTA_NW, DELTA_ZERO},
  {DELTA_S, DELTA_E, DELTA_W, DELTA_N, 
   DELTA_SE, DELTA_SW, DELTA_NE, DELTA_NW, DELTA_ZERO},
  {DELTA_ZERO},
  {DELTA_ZERO},
  {DELTA_SW, DELTA_SE, DELTA_ZERO},
  {DELTA_SSW, DELTA_SSE, DELTA_SWW, DELTA_SEE,
   DELTA_NWW, DELTA_NEE, DELTA_NNW, DELTA_NNE, DELTA_ZERO},
  {DELTA_SE, DELTA_SW, DELTA_NE, DELTA_NW, DELTA_ZERO},
  {DELTA_S, DELTA_E, DELTA_W, DELTA_N, DELTA_ZERO},
  {DELTA_S, DELTA_E, DELTA_W, DELTA_N, 
   DELTA_SE, DELTA_SW, DELTA_NE, DELTA_NW, DELTA_ZERO},
  {DELTA_S, DELTA_E, DELTA_W, DELTA_N, 
   DELTA_SE, DELTA_SW, DELTA_NE, DELTA_NW, DELTA_ZERO},
};   

const SquareDelta PawnPush[2] = {
  DELTA_N, DELTA_S
};


////
//// Functions
////

char piece_type_to_char(PieceType pt, bool upcase = false) {
  char pieceChars[] = " pnbrqk";
  if(upcase)
    return pieceChars[pt] + ('A' - 'a');
  else
    return pieceChars[pt];
}

bool piece_is_ok(Piece pc) {
  return
    piece_type_is_ok(type_of_piece(pc)) &&
    color_is_ok(color_of_piece(pc));
}

bool piece_type_is_ok(PieceType pc) {
  return pc >= PAWN && pc <= KING;
}
