// Copyright (C) 2004-2007 Tord Romstad

// This file is part of Glaurung.
//
// Glaurung is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//  
// Glaurung is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Glaurung; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#if !defined(SEARCH_H_INCLUDED)
#define SEARCH_H_INCLUDED

////
//// Includes
////

#include "depth.h"
#include "history.h"
#include "lock.h"
#include "movegen.h"
#include "ply.h"
#include "position.h"
#include "value.h"


////
//// Types
////

/// The SearchStack struct keeps track of the information we need to remember
/// from nodes shallower and deeper in the tree during the search.  Each
/// search thread has its own array of SearchStack objects, indexed by the
/// current ply.

struct SearchStack {
  Move pv[PLY_MAX];
  Move currentMove;
  Value currentMoveCaptureValue;
  Move mateKiller, killer1, killer2;
  Move threatMove;
  Depth reduction;
};


////
//// Global variables
////

extern int ActiveThreads;

extern Lock SMPLock;

// Perhaps better to make H local, and pass as parameter to PickMove?
extern History H;  


////
//// Prototypes
////

extern void init_threads();
extern void stop_threads();
extern void think(const Position& pos, bool infinite, bool ponder, int time,
                  int increment, int movesToGo, int maxDepth, int maxNodes);
extern uint64 nodes();
extern bool position_is_quiescent(const Position& pos);


#endif // !defined(SEARCH_H_INCLUDED)
